import { Directive, Input, HostListener, ElementRef, Pipe, forwardRef, InjectionToken, Inject, EventEmitter, Output, NgModule } from '@angular/core';
import { NG_VALIDATORS } from '@angular/forms';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/map';
import 'rxjs/add/observable/fromPromise';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const CEP_MASK = '_____-___';
const CNPJ_MASK = '__.___.___/____-__';
const CPF_MASK = '___.___.___-__';
const DATA_MASK = 'DD/MM/AAAA';
class StringFormatter {
    /**
     * Aplica uma máscara a uma string
     *
     * @param {?} mask A máscara desejada
     * @param {?=} maskCharsArray (Opcional) um array contendo as strings da máscara, por exemplo
     * a máscara DD/MM/AAAA tem que passar o maskCharsArray como ['D', 'M', 'A'], para máscaras
     * como __/__/____ esse valor de maskCharsArray é opcional, ficando igual a ['_']
     */
    constructor(mask, maskCharsArray) {
        this.maskCharsSet = new Set();
        this.placeholderChar = '¬';
        this.emptyString = '';
        this.caretPosition = 0;
        this.unchangeMask = mask;
        if (!maskCharsArray || maskCharsArray.length <= 0) {
            maskCharsArray = [this.getPlaceholderChar(mask)];
        }
        for (const /** @type {?} */ chr of maskCharsArray) {
            const /** @type {?} */ re = new RegExp(chr, 'gi');
            mask = mask.replace(re, this.placeholderChar);
        }
        this.formattedMaskArray = mask.split(this.emptyString);
        this.unchangeMask.split(this.emptyString).forEach((item) => this.maskCharsSet.add(item));
    }
    /**
     * Retorna um texto formatado usando uma máscara
     * @param {?} txt Texto a ser formatado
     * @param {?} mask máscara
     * @param {?=} maskChars (opcional) caracteres da máscara, por exemplo:
     * A máscara DD/MM/AAAA tem que passar o maskChars como ['D', 'M', 'A'], para máscaras
     * como __/__/____ o valor de maskChars é opcional pois é obtido automaticamente, ficando
     * igual a ['_']
     * @return {?}
     */
    static maskedFormatter(txt, mask, maskChars = []) {
        const /** @type {?} */ strFmt = StringFormatter.getStringFormatter(mask, maskChars);
        return strFmt.format(txt);
    }
    /**
     * Retorna uma instância do StringFormatter
     * @param {?} mask máscara
     * @param {?=} maskChars (opcional) caracteres da máscara, por exemplo:
     * A máscara DD/MM/AAAA tem que passar o maskChars como ['D', 'M', 'A'], para máscaras
     * como __/__/____ o valor de maskChars é opcional pois é obtido automaticamente, ficando
     * igual a ['_']
     * @return {?}
     */
    static getStringFormatter(mask, maskChars = []) {
        return new StringFormatter(mask, maskChars);
    }
    /**
     * Remove a formatação de uma máscara de um texto
     * @param {?} txt Texto a ser formatado
     * @param {?} mask máscara
     * @param {?=} maskChars (opcional) caracteres da máscara, por exemplo:
     * A máscara DD/MM/AAAA tem que passar o maskChars como ['D', 'M', 'A'], para máscaras
     * como __/__/____ o valor de maskChars é opcional pois é obtido automaticamente, ficando
     * igual a ['_']
     * @return {?}
     */
    static clearFormat(txt, mask, maskChars = []) {
        const /** @type {?} */ strFmt = StringFormatter.getStringFormatter(mask, maskChars);
        return strFmt.stripMask(txt);
    }
    /**
     * Retorna um texto sem a formatação da máscara
     * @param {?} maskedTxt Texto formatado
     * @return {?}
     */
    stripMask(maskedTxt) {
        // criando uma cópia do maskedTxt e removendo a marcação
        // criando uma cópia do maskedTxt e removendo a marcação
        return [...maskedTxt.split(this.emptyString)].
            filter(char => (!this.maskCharsSet.has(char))).join(this.emptyString);
    }
    /**
     * Retorna a posição que o cursor deve ter
     * @return {?}
     */
    getCaretPosition() {
        return this.caretPosition;
    }
    /**
     * Retorna o texto formatado
     * @param {?} txt texto a ser formatado
     * @return {?}
     */
    format(txt) {
        this.caretPosition = 0;
        const /** @type {?} */ inputArray = this.stripMask(txt).split(this.emptyString);
        let /** @type {?} */ formattedStr = this.formattedMaskArray.map((char, index) => {
            if (char !== this.placeholderChar) {
                // this.caretPosition += 1;
                // this.caretPosition += 1;
                return char;
            }
            if (inputArray.length === 0) {
                return char;
            }
            // this.caretPosition += 1;
            // this.caretPosition += 1;
            return inputArray.shift();
        }).join(this.emptyString);
        const /** @type {?} */ idx = formattedStr.indexOf(this.placeholderChar);
        if (idx !== -1) {
            this.caretPosition = idx;
            formattedStr = formattedStr.substring(0, idx) + this.unchangeMask.substr(idx);
        }
        else {
            this.caretPosition = this.unchangeMask.length;
        }
        return formattedStr;
    }
    /**
     * @param {?} str
     * @return {?}
     */
    getPlaceholderChar(str) {
        const /** @type {?} */ strCountMap = new Map();
        let /** @type {?} */ maxKey = this.emptyString;
        for (const /** @type {?} */ key of str) {
            if (!strCountMap.has(key)) {
                strCountMap.set(key, 1);
            }
            else {
                strCountMap.set(key, strCountMap.get(key) + 1);
            }
            if (maxKey === this.emptyString || strCountMap.get(maxKey) < strCountMap.get(key)) {
                maxKey = key;
            }
        }
        return maxKey;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MaskPatternDirective {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
        // tslint:disable-next-line:no-input-rename
        this.showPlaceholder = true;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.showPlaceholder && !this.el.nativeElement.placeholder) {
            this.el.nativeElement.placeholder = this.pattern;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        const /** @type {?} */ input = (/** @type {?} */ (event.target));
        let /** @type {?} */ oldStart = input.selectionStart;
        let /** @type {?} */ oldEnd = input.selectionEnd;
        // const oldInptLen = input.value.length;
        let /** @type {?} */ maskChars = [];
        if (this.maskChars) {
            maskChars = maskChars.concat(this.maskChars.split(','));
        }
        const /** @type {?} */ strFmt = StringFormatter.getStringFormatter(this.pattern, maskChars);
        input.value = strFmt.format(input.value);
        if (oldEnd + 1 === strFmt.getCaretPosition()) {
            oldStart += 1;
            oldEnd += 1;
        }
        input.selectionStart = oldStart;
        input.selectionEnd = oldEnd;
    }
}
MaskPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsMaskPattern]'
            },] },
];
/** @nocollapse */
MaskPatternDirective.ctorParameters = () => [
    { type: ElementRef, },
];
MaskPatternDirective.propDecorators = {
    "pattern": [{ type: Input, args: ['ngBrToolsMaskPattern',] },],
    "showPlaceholder": [{ type: Input, args: ['ngBrToolsShowPlacholder',] },],
    "maskChars": [{ type: Input, args: ['ngBrToolsMaskChars',] },],
    "onInput": [{ type: HostListener, args: ['keyup', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MaskPatternPipe {
    /**
     * @param {?} value
     * @param {?} mask
     * @return {?}
     */
    transform(value, mask) {
        return StringFormatter.maskedFormatter(value, mask);
    }
}
MaskPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsMaskPatternPipe'
            },] },
];
/** @nocollapse */
MaskPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CpfPatternDirective extends MaskPatternDirective {
    /**
     * @return {?}
     */
    ngOnInit() {
        this.pattern = CPF_MASK;
        super.ngOnInit();
    }
}
CpfPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCpfPattern]'
            },] },
];
/** @nocollapse */
CpfPatternDirective.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CpfPatternPipe extends MaskPatternPipe {
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        return super.transform(value, CPF_MASK);
    }
}
CpfPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCpfPatternPipe'
            },] },
];
/** @nocollapse */
CpfPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/** @enum {number} */
const Documento = {
    CPF: 0,
    CNPJ: 1,
};
Documento[Documento.CPF] = "CPF";
Documento[Documento.CNPJ] = "CNPJ";
class BrValidator {
    /**
     * @param {?} cpf
     * @return {?}
     */
    static validaCpf(cpf) {
        const /** @type {?} */ brValidatorUtils = new BrValidator(cpf, Documento.CPF);
        return brValidatorUtils.isValid();
    }
    /**
     * @param {?} cnpj
     * @return {?}
     */
    static validaCnpj(cnpj) {
        const /** @type {?} */ brValidatorUtils = new BrValidator(cnpj, Documento.CNPJ);
        return brValidatorUtils.isValid();
    }
    /**
     * @param {?} numero
     * @param {?} tipo
     */
    constructor(numero, tipo) {
        this.documento = {
            tipo: tipo,
            numeroCompleto: numero,
            numero: '',
            digito1: '',
            digito2: ''
        };
        if (!numero || numero.length === 0) {
            this.documento.numeroCompleto = '';
            return;
        }
        this.documento.numero = numero.substr(0, numero.length - 2);
        this.documento.digito1 = numero.substr(numero.length - 2, 1);
        this.documento.digito2 = numero.substr(numero.length - 1, 1);
    }
    /**
     * @return {?}
     */
    isValid() {
        switch (this.documento.tipo) {
            case Documento.CPF:
                return this.isValidCpf();
            case Documento.CNPJ:
                return this.isValidCnpj();
        }
    }
    /**
     * @return {?}
     */
    isValidCpf() {
        if (this.documento.numeroCompleto.length !== 11) {
            return false;
        }
        if (this.getInvalidsCpfs().has(this.documento.numeroCompleto)) {
            return false;
        }
        return this.documento.numeroCompleto ===
            (this.documento.numero + this.getDigitoVerificador());
    }
    /**
     * @return {?}
     */
    isValidCnpj() {
        if (this.documento.numeroCompleto.length !== 14) {
            return false;
        }
        if (this.getInvalidsCnpjs().has(this.documento.numeroCompleto)) {
            return false;
        }
        return this.documento.numeroCompleto ===
            (this.documento.numero + this.getDigitoVerificador());
    }
    /**
     * @return {?}
     */
    getDigitoVerificador() {
        const /** @type {?} */ digito1 = this.calculaDigitoVerificador(this.documento.numero);
        const /** @type {?} */ digito2 = this.calculaDigitoVerificador(this.documento.numero + digito1);
        return digito1 + digito2;
    }
    /**
     * @param {?} numero
     * @return {?}
     */
    calculaDigitoVerificador(numero) {
        let /** @type {?} */ tblPesos = [];
        // Pegando a ordem inversa da tabela de pesos
        // junto com a ordem inversa so número, fica mais
        // simples o cálculo dos digitos verificadores
        // Pegando a ordem inversa da tabela de pesos
        // junto com a ordem inversa so número, fica mais
        // simples o cálculo dos digitos verificadores
        switch (this.documento.tipo) {
            case Documento.CPF:
                tblPesos = this.getCpfTable().reverse();
                break;
            case Documento.CNPJ:
                tblPesos = this.getCnpjTable().reverse();
                break;
        }
        // Faz o somatório dos digitos * valor da posição tabela de pesos
        const /** @type {?} */ somatorio = numero.split('').reverse().map((char, i) => {
            return Number(char) * tblPesos[i];
        }).reduce((sum, current) => sum + current);
        // Divide o somatório por 11 e obtém o resto
        const /** @type {?} */ resto = somatorio % 11;
        // Se o resto for menor que 2, então o dígito é zero
        // senão o digito vai ser igual ao 11 - resto
        // Então se o 11 - resto > 9  o digito = 0 (que dá no mesmo)
        let /** @type {?} */ digito = 11 - resto;
        if (digito > 9) {
            digito = 0;
        }
        return String(digito);
    }
    /**
     * @return {?}
     */
    getCpfTable() {
        return [11, 10, 9, 8, 7, 6, 5, 4, 3, 2];
    }
    /**
     * @return {?}
     */
    getCnpjTable() {
        return [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];
    }
    /**
     * @return {?}
     */
    getInvalidsCpfs() {
        return new Set(['00000000000', '11111111111', '22222222222', '33333333333',
            '44444444444', '55555555555', '66666666666', '77777777777', '88888888888',
            '99999999999']);
    }
    /**
     * @return {?}
     */
    getInvalidsCnpjs() {
        return new Set(['00000000000000', '11111111111111', '22222222222222', '33333333333333',
            '44444444444444', '55555555555555', '66666666666666', '77777777777777',
            '88888888888888', '99999999999999']);
    }
}

// class BrValidatotUtils {
//   constructor(private str: string) {}
//   isValid(): boolean {
//     return this.checkCanBeValid(this.str, 'cpf') ||
//       this.checkCanBeValid(this.str, 'cnpj');
//   }
//   calculaDigitoVerificador(str: string): string {
//     let tblPesos: number[] = [];
//     let digito: number;
//     let somatorio = 0;
//     // Definindo que tabela de pesos
//     // devo usar, cnpj ou cpf
//     if (str.length === 9) {
//       // CPF
//       tblPesos = [10, 9, 8, 7, 6, 5, 4, 3, 2];
//     } else if (str.length === 10 ) {
//       // CPF + primeiro digito verificador
//       tblPesos = [11, 10, 9, 8, 7, 6, 5, 4, 3, 2];
//     } else if (str.length === 12) {
//       // CNPJ
//       tblPesos = [5, 4, 3, 2, 9, 8, 7, 6, 5 , 4, 3, 2];
//     } else if (str.length === 13) {
//       // CNPJ + primeiro dígito verificador
//       tblPesos = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5 , 4, 3, 2];
//     }
//     // Faz o somatório dos digitos * posição tabela de pesos
//     let i = 0;
//     for (const p of tblPesos) {
//       somatorio += p * Number(str.charAt(i));
//       i++;
//     }
//     // Divide o somatório por 11 e obtém o resto
//     const resto = somatorio % 11;
//     // Se o resto for menor que 2, então o dígito é zero
//     // sentão  o ditigo vai ser igual ao 11 - resto
//     // Então se o 11 - resto > 9  o digito = 0 (que dá no mesmo)
//     digito = 11 - resto;
//     if (digito > 9) {
//         digito = 0;
//     }
//     return digito.toString();
//     }
//   removeDigitosVerificador(): string {
//     return this.str.substring(0, this.str.length - 2);
//   }
//   private checkCanBeValid(value: string, opt: string): boolean {
//     if (value == null || value.length === 0) {
//       return false;
//     }
//     switch (opt) {
//       case 'cpf':
//         return this.canBeValidCpf(value);
//       case 'cnpj':
//         return this.canBeValidCnpj(value);
//     }
//     return false;
//   }
//   private canBeValidCpf(cpf: string): boolean {
//     if (cpf.length !== 11) {
//       return false;
//     }
//     if ((cpf === '00000000000') ||
//         (cpf === '11111111111') ||
//         (cpf === '22222222222') ||
//         (cpf === '33333333333') ||
//         (cpf === '44444444444') ||
//         (cpf === '55555555555') ||
//         (cpf === '66666666666') ||
//         (cpf === '77777777777') ||
//         (cpf === '88888888888') ||
//         (cpf === '99999999999')) {
//       return false;
//     }
//     return true;
//   }
//   private canBeValidCnpj(cnpj: string): boolean {
//     if (cnpj.length !== 14) {
//       return false;
//     }
//     if ( '00000000000000') ||
//          '11111111111111') ||
//          '22222222222222') ||
//          '33333333333333') ||
//          '44444444444444') ||
//          '55555555555555') ||
//          '66666666666666') ||
//          '77777777777777') ||
//          '88888888888888') ||
//          '99999999999999')) {
//       return false;
//     }
//     return true;
//   }
// }

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// tslint:disable-next-line:directive-class-suffix
class CPFValidator {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        if (!(c.value && c.value.length > 0)) {
            // Nenhum valor passado, se o campo for requerido deve-se usar
            // o validator.required, esse só valida o cpf
            // Nenhum valor passado, se o campo for requerido deve-se usar
            // o validator.required, esse só valida o cpf
            return null;
        }
        // Limpando qualquer caracter de formatação, só importa os números
        const /** @type {?} */ value = c.value.split('').filter((char) => !isNaN(Number(char))).join('');
        if (BrValidator.validaCpf(value)) {
            return null;
        }
        return { 'cpf': true };
    }
}
CPFValidator.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCpfValidator][ngModel],[ngBrToolsCpfValidator][formControl],[ngBrToolsCpfValidator][formControlName]',
                providers: [
                    {
                        provide: NG_VALIDATORS,
                        useExisting: forwardRef(() => CPFValidator),
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
CPFValidator.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CnpjPatternDirective extends MaskPatternDirective {
    /**
     * @return {?}
     */
    ngOnInit() {
        this.pattern = CNPJ_MASK;
        super.ngOnInit();
    }
}
CnpjPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCnpjPattern]'
            },] },
];
/** @nocollapse */
CnpjPatternDirective.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CnpjPatternPipe extends MaskPatternPipe {
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        return super.transform(value, CNPJ_MASK);
    }
}
CnpjPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCnpjPatternPipe'
            },] },
];
/** @nocollapse */
CnpjPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// tslint:disable-next-line:directive-class-suffix
class CNPJValidator {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        if (!(c.value && c.value.length > 0)) {
            return null;
        }
        // Limpando qualquer caracter de formatação, só importa os números
        const /** @type {?} */ value = c.value.split('').filter((char) => !isNaN(Number(char))).join('');
        if (BrValidator.validaCnpj(value)) {
            return null;
        }
        return { 'cnpj': true };
    }
}
CNPJValidator.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCnpjValidator][ngModel],[ngBrToolsCnpjValidator][formControl],[ngBrToolsCnpjValidator][formControlName]',
                providers: [
                    {
                        provide: NG_VALIDATORS,
                        useExisting: forwardRef(() => CNPJValidator),
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
CNPJValidator.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let ESTADO_SERVICE = new InjectionToken('estado.service');
let EstadoServiceFactory = (estadoService) => {
    estadoService.init();
    return estadoService;
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class LstEstadoDirective {
    /**
     * @param {?} el
     * @param {?} estadoService
     */
    constructor(el, estadoService) {
        this.el = el;
        // tslint:disable-next-line:no-input-rename
        this.orderBy = 'nome';
        // tslint:disable-next-line:no-output-rename
        this.onEstados = new EventEmitter();
        this._estadoService = estadoService;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // Carregando os estados
        let /** @type {?} */ resp = this._estadoService.buscaEstados();
        if (resp instanceof Promise) {
            resp = Observable.fromPromise((/** @type {?} */ (resp)));
        }
        (/** @type {?} */ (resp)).map((estados) => {
            return this._sortEstados(estados, this.orderBy);
        }).subscribe((e) => this.onEstados.emit(e));
    }
    /**
     * @param {?} lst
     * @param {?=} por
     * @return {?}
     */
    _sortEstados(lst, por = 'nome') {
        if (lst.length <= 1) {
            return lst;
        }
        if (por === 'nome') {
            return lst.slice(0).sort((e1, e2) => {
                if (e1.nome === e2.nome) {
                    return 0;
                }
                return (e1.nome < e2.nome) ? -1 : 1;
            });
        }
        else if (por === 'sigla') {
            return lst.slice(0).sort((e1, e2) => {
                if (e1.sigla === e2.sigla) {
                    return 0;
                }
                return (e1.sigla < e2.sigla) ? -1 : 1;
            });
        }
    }
}
LstEstadoDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsListEstados]'
            },] },
];
/** @nocollapse */
LstEstadoDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: undefined, decorators: [{ type: Inject, args: [ESTADO_SERVICE,] },] },
];
LstEstadoDirective.propDecorators = {
    "orderBy": [{ type: Input, args: ['ngBrToolsOrderBy',] },],
    "onEstados": [{ type: Output, args: ['ngBrToolsListEstados',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Estado {
    /**
     * @param {?=} nome
     * @param {?=} sigla
     * @param {?=} bandeira
     */
    constructor(nome, sigla, bandeira) {
        this.nome = nome;
        this.sigla = sigla;
        this.bandeira = bandeira;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CepPatternDirective extends MaskPatternDirective {
    /**
     * @return {?}
     */
    ngOnInit() {
        this.pattern = CEP_MASK;
        super.ngOnInit();
    }
}
CepPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCepPattern]'
            },] },
];
/** @nocollapse */
CepPatternDirective.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CepPatternPipe extends MaskPatternPipe {
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        return super.transform(value, CEP_MASK);
    }
}
CepPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCepPatternPipe'
            },] },
];
/** @nocollapse */
CepPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let CEP_SERVICE = new InjectionToken('cep.service');
let CepServiceFactory = (cepService) => {
    cepService.init();
    return cepService;
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CepSearchDirective {
    /**
     * @param {?} cepEl
     * @param {?} cepService
     */
    constructor(cepEl, cepService) {
        this.cepEl = cepEl;
        // tslint:disable-next-line:no-input-rename
        this.onEndereco = new EventEmitter();
        this.onError = new EventEmitter();
        this._cepService = cepService;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onInput(e) {
        const /** @type {?} */ cepInput = (/** @type {?} */ (e.target));
        let /** @type {?} */ pattern;
        if (cepInput.hasAttribute('ngBrToolsMaskPattern')) {
            pattern = cepInput.getAttribute('ngBrToolsMaskPattern');
        }
        else {
            pattern = CEP_MASK;
        }
        const /** @type {?} */ cepClear = StringFormatter.clearFormat(cepInput.value, pattern);
        if (cepClear.length === 8) {
            this.doSearch(cepClear);
        }
    }
    /**
     * @param {?} cep
     * @return {?}
     */
    doSearch(cep) {
        const /** @type {?} */ resp = this._cepService.buscaCep(cep);
        if (resp instanceof Observable) {
            (/** @type {?} */ (resp)).subscribe((e) => {
                this.onEndereco.emit(e);
            }, (err) => {
                this.onError.emit(err);
            });
        }
        else {
            (/** @type {?} */ (resp))
                .then((e) => this.onEndereco.emit(e))
                .catch((err) => {
                this.onError.emit(err);
            });
        }
    }
}
CepSearchDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCepSearch]'
            },] },
];
/** @nocollapse */
CepSearchDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: undefined, decorators: [{ type: Inject, args: [CEP_SERVICE,] },] },
];
CepSearchDirective.propDecorators = {
    "onEndereco": [{ type: Output },],
    "onError": [{ type: Output },],
    "onInput": [{ type: HostListener, args: ['input', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Endereco {
    /**
     * @param {?=} endereco
     * @param {?=} bairro
     * @param {?=} cep
     * @param {?=} cidade
     * @param {?=} complemento
     * @param {?=} complemento2
     * @param {?=} uf
     * @param {?=} id
     */
    constructor(endereco, bairro, cep, cidade, complemento, complemento2, uf, id) {
        this.endereco = endereco;
        this.bairro = bairro;
        this.cep = cep;
        this.cidade = cidade;
        this.complemento = complemento;
        this.complemento2 = complemento2;
        this.uf = uf;
        this.id = id;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class DataPatternDirective extends MaskPatternDirective {
    /**
     * @return {?}
     */
    ngOnInit() {
        this.pattern = DATA_MASK;
        this.maskChars = 'D,M,A';
        super.ngOnInit();
    }
}
DataPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsDataPattern]'
            },] },
];
/** @nocollapse */
DataPatternDirective.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class DataPatternPipe {
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        return StringFormatter.maskedFormatter(value, DATA_MASK, ['D', 'M', 'A']);
    }
}
DataPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'data-pattern'
            },] },
];
/** @nocollapse */
DataPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NgBrToolsValidators {
    /**
     * @param {?} control
     * @return {?}
     */
    static cpfValidator(control) {
        const /** @type {?} */ validator = new CPFValidator();
        return validator.validate(control);
    }
    /**
     * @param {?} control
     * @return {?}
     */
    static cnpjValidator(control) {
        const /** @type {?} */ validator = new CNPJValidator();
        return validator.validate(control);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NgBrToolsModule {
}
NgBrToolsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    MaskPatternDirective,
                    CepPatternDirective,
                    CepSearchDirective,
                    CpfPatternDirective,
                    CnpjPatternDirective,
                    DataPatternDirective,
                    LstEstadoDirective,
                    MaskPatternPipe,
                    CepPatternPipe,
                    CpfPatternPipe,
                    CnpjPatternPipe,
                    DataPatternPipe,
                    CPFValidator,
                    CNPJValidator,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [],
                exports: [
                    MaskPatternDirective,
                    CepPatternDirective,
                    CepSearchDirective,
                    CpfPatternDirective,
                    CnpjPatternDirective,
                    DataPatternDirective,
                    LstEstadoDirective,
                    MaskPatternPipe,
                    CepPatternPipe,
                    CpfPatternPipe,
                    CnpjPatternPipe,
                    DataPatternPipe,
                    CPFValidator,
                    CNPJValidator,
                ],
            },] },
];
/** @nocollapse */
NgBrToolsModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { MaskPatternDirective, MaskPatternPipe, CpfPatternDirective, CpfPatternPipe, CPFValidator, CnpjPatternDirective, CnpjPatternPipe, CNPJValidator, LstEstadoDirective, ESTADO_SERVICE, EstadoServiceFactory, Estado, CepPatternDirective, CepPatternPipe, CepSearchDirective, CEP_SERVICE, CepServiceFactory, Endereco, DataPatternDirective, DataPatternPipe, NgBrToolsValidators, NgBrToolsModule };
//# sourceMappingURL=ng-br-tools.js.map
